# $Id: AMAVIS.pm,v 1.31.2.5 2002/05/11 15:05:26 bengen Exp $

#
# AMAVIS main module
#

# Documentation of the message description hash:
# ----------------------------------------------
# status: $
# directory: $
# sender: $
# recipients: @
# virus: %
#   $av: @         (For each $av, found viruses are inserted here)
# policy: %
#   $policy: @     (For each $policy module, found offenses are inserted
#                   here)
# unpacked_size    (Counters for size so-far...
# unpacked_files             ... and number of files)

package AMAVIS;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

require Exporter;
@AMAVIS::ISA = qw(Exporter);
# Currently these symbols are just taken from syslog(3). This may
# change in the future.
@AMAVIS::EXPORT = qw(
		     get_secure_filename cmd_pipe

		     $cfg_maxlevels
		     $cfg_maxfiles
		     $cfg_maxspace
		    );

use AMAVIS::Logging;

use Config::IniFiles;
use Sys::Syslog qw(:DEFAULT setlogsock);

use POSIX qw(strftime geteuid setuid uname setsid setuid setgid);
use POSIX ":sys_wait_h";

use File::Copy;
use File::Path;
use AMAVIS::Magic;
use IO::File;
use IO::Handle;

# Message digesting modules

use vars qw(
	    $umask
	    $conffile

	    $directory
	    $cfg
	    $mta
	    $mta_in $mta_out 

	    @av @policy

	    $cfg_syslog

	    $cfg_syslog_facility $cfg_syslog_loglevel
	    $cfg_logfile $cfg_logfile_loglevel

	    $log_fd

	    $cfg_maxlevels
	    $cfg_maxfiles
	    $cfg_maxspace

	    $log

	    @extractors
	    %type_extractors
	    @notifiers

	    $magic
	    $magicfile

	    $cfg_uid
	    $cfg_gid

	    $cfg_quarantine_dir

	   );

# For parsing message...
use vars qw(
	    %selfextract %filedata
	   );

$magicfile='/usr/share/amavis/magic.mime';
$conffile = '/etc/amavis/amavis.conf';
# $conffile = 'test/amavis.conf.test';

sub init {
  my $self = shift;
  my $args = shift;
  # Read config file
  my %options = @_;

  if (defined $options{'configfile'}) {
    $conffile=$options{'configfile'}
  }

  $cfg = new Config::IniFiles(-file => "$conffile") or do {
    die("Unable to open config file $conffile: ".
	join "\n",@Config::IniFiles::errors);
  };

  AMAVIS::Logging::init_logging();

  $umask = $cfg->val('global', 'umask');
  if (defined $umask) {
    umask $umask;
  }
  else {
    umask '002';
  }

  $cfg_uid = $cfg->val('security', 'uid');
  $cfg_gid = $cfg->val('security', 'gid');

  # If UID, GID are not numeric or not set at all...
  if ((! defined $cfg_uid) or ($cfg_uid !~ /^\d+$/)) {
    $cfg_uid = getpwnam($cfg_uid) || $>;
  }
  if ((! defined $cfg_gid) or ($cfg_gid !~ /^\d+$/)) {
    $cfg_gid = getgrnam($cfg_gid) || $);
  }

  $cfg_maxlevels = $cfg->val('security', 'maxlevels');
  $cfg_maxfiles = $cfg->val('security', 'maxfiles');
  $cfg_maxspace = $cfg->val('security', 'maxspace');

  # kB...TB to bytes for maxspace
  if ($cfg_maxspace =~ /^\s*(\d+)\s*([KkMmGgTt])[Bb]?\s*$/ ) {
    my $num = $1; 
    foreach ($2) {
      /[Kk]/ && do {
	$cfg_maxspace = $num * 1024;
	last;
      };
      /[Mm]/ && do {
	$cfg_maxspace = $num * 1048576;
	last;
      };
      /[Gg]/ && do {
	$cfg_maxspace = $num * 1073741824;
	last;
      };
      /[Tt]/ && do {
	$cfg_maxspace = $num * 1099511627776;
	last;
      };
    }
  }

  # Quarantine path
  $cfg_quarantine_dir = ($cfg->val('paths', 'quarantine') or 'no');

  # MTA initialization
  $mta = 'AMAVIS::MTA::'.$cfg->val('global', 'mail-transfer-agent');
  eval "use $mta $VERSION;1" or do {
    writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't load $mta");
    die __PACKAGE__.": Couldn't load $mta";
  };
  $mta->init($args) or do {
    writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't init $mta");
    die __PACKAGE__.": Couldn't init $mta";
  };

  # Set up file type detection.
  $magic=AMAVIS::Magic->new($magicfile) or do {
    writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't init AMAVIS::Magic (File::MMagic)");
    die __PACKAGE__.": Couldn't init AMAVIS::Magic (File::MMagic)";
  };

  # for the time being: Work around missing methods if necessary.
  eval {
    $magic->removeSpecials();
    $magic->removeFileExts();
  };
  if ($@) {
    writelog($args,LOG_DEBUG, "Old version of File::MMagic. $@");
    $magic->{SPECIALS} = {};
    $magic->{FILEEXTS} = {};
  }
  $magic->read_magic_file;

  # Used for get_secure_filename
  $$args{'filename_counter'}=0;

  # Extractors initialization
  @extractors=split / *, */,$cfg->val('global','extractors');
  unless (defined $extractors[0]) {
    shift @extractors;
  }
  foreach my $extractor (@extractors) {
    $extractor = 'AMAVIS::Extract::'.$extractor;
    eval "use $extractor $VERSION;1" or do {
      writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't load $extractor");
      die __PACKAGE__.": Couldn't load $extractor";
    };
    $extractor->init($args,\%type_extractors) or do {
      writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't init $extractor");
      die __PACKAGE__.": Couldn't init $extractor";
    };
  }

#   # Policy filters initialization
#   @policy = $cfg->val('global', 'policy');
#   unless (defined $policy[0]) {
#     shift @policy;
#   }
#   foreach my $policy (@policy) {
#     $policy = 'AMAVIS::Policy::'.$policy;
#     eval "use $policy $VERSION;1" or do {
#       writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't load $policy");
#       die __PACKAGE__.": Couldn't load $policy";
#     };
#     $policy->init($args) or do {
#       writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't init $policy");
#       die __PACKAGE__.": Couldn't init $policy";
#     };
#   }

  # Virus scanners initialization
  @av = $cfg->val('global', 'virus-scanner');
  unless (defined $av[0]) {
    shift @av;
  }
  foreach my $av (@av) {
    $av =~ /^\s*(.*)\s*$/;
  }
  foreach my $av (@av) {
    $av = 'AMAVIS::AV::'.$av;
    eval "use $av $VERSION;1" or do {
      writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't load $av");
      die __PACKAGE__.": Couldn't load $av";
    };
    $av->init($args) or do {
      writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't init $av");
      die __PACKAGE__.": Couldn't init $av";
    };
  }

  @notifiers=split / *, */,$cfg->val('global','notifiers');
  unless (defined $notifiers[0]) {
    shift @notifiers;
  }
  foreach my $notifier (@notifiers) {
    $notifier = 'AMAVIS::Notify::'.$notifier;
    eval "use $notifier $VERSION;1" or do {
      writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't load $notifier");
      die __PACKAGE__.": Couldn't load $notifier";
    };
    $notifier->init($args) or do {
      writelog($args,LOG_CRIT, __PACKAGE__.": Couldn't init $notifier");
      die __PACKAGE__.": Couldn't init $notifier";
    };
  }

  unless (defined @{$$args{'virus_scanners'}}) {
    push @{$$args{'virus_scanners'}}, '(none)';
  }

  writelog($args,LOG_DEBUG, __PACKAGE__." initialized");
  my $uid = getpwuid($<) || "Kilroy";
  my $euid = getpwuid($>) || "Kilroy";
  writelog($args,LOG_DEBUG, __PACKAGE__.": Running as UID/EUID $uid($<)/$euid($>)");
}

# "Universal" extraction routine
# Determine mime type of each extracted file, then call extract
# routine for that mime type
sub extract {
  my $args=shift;

  my $newfilesfound;
  my $level;

  for ($level=0; $level<=$cfg_maxlevels; $level++) {
    $newfilesfound=0;
    foreach my $filename (keys % {$$args{'contents'}}) {
      unless (defined $ {$ {$$args{'contents'}}{$filename}}{'type'}) {
	# The filetype isn't known yet, this means the file is new.
	# First determine the filetype, then.
	my $filetype=$magic->checktype_filename("$$args{'directory'}/parts/$filename");
	# Store the filetype.
	$ {$ {$$args{'contents'}}{$filename}}{'type'} = $filetype;
	writelog($args,LOG_INFO, "File $filename is type $filetype");
	if (defined $type_extractors{$filetype}) {
	  # We know how to unpack this sucker...
	  $type_extractors{$filetype}->extract($args, $filename) or do {
	    writelog($args,LOG_ERR, "Error while unpacking $filename");
	    return 0;
	  };
	  $newfilesfound=1;
	}
	else {
	  writelog($args,LOG_INFO, "Not attempting to unpack $filename");
	}
      }
    }
    last unless ($newfilesfound);
  }
  if ($level >= $cfg_maxlevels) {
    writelog($args,LOG_ERR,"Mail has too many archive levels");
    return 0;
  }
  return 1;
}

sub get_secure_filename {
  my $args=shift;
  return sprintf("%.8x",$$args{'filename_counter'}++);
}

sub process_message {
  my $self = shift;
  my $args = shift;

  $$args{'status'} = 'unknown';
  # Create temporary directory and have MTA module write the message
  # there, if necessary.

  unless (defined $$args{'directory'}) {
    $mta->get_directory($args) or do {
      writelog($args,LOG_CRIT, "Error: Couldn't get directory for extracting");
      return 0;
    };
  }
  writelog($args,LOG_INFO, "Unpacking message in $$args{'directory'}");

  if ($$args{'sender'} eq '<>') {
    $$args{'sender'} = '';
  }

  # chdir to tmp directory (which has to be writable!)
  my $tempdir = $cfg->val('paths', 'unpack');
  chdir $tempdir or do {
    writelog($args,LOG_CRIT, "Couldn't chdir to $tempdir");
    $mta->freeze_message($args);
    return 0;
  };

  my $securename=get_secure_filename($args);

  # Initialize unpacked_size to size of mail file...
  my @stat = stat("$$args{'directory'}/email.txt");
  $$args{'unpacked_size'} = $stat[7];
  # ...and unpacked_files to 1.
  $$args{'unpacked_files'} = 1;

  # The first file to be analyzed is the mail file
  link( "$$args{'directory'}/email.txt",
	"$$args{'directory'}/parts/$securename") 
    or do {
      writelog($args,LOG_CRIT, "Unable to link parts/$securename to email.txt");
      $mta->freeze_message($args);
      return 0;
    };

  # Add filename to contents field of message.
  $ {$$args{'contents'}}{$securename} = {};

  extract($args,$securename) or do {
    writelog($args,LOG_CRIT, "Error while unpacking message");
    $mta->freeze_message($args);
    return 0;
  };

#   foreach my $policy (@policy) {
#     $policy->scan($args) or
#       writelog($args,LOG_CRIT,"Error while scanning for policy");
#   }

  foreach my $av (@av) {
    $av->scan($args) or do {
      writelog($args,LOG_CRIT,"Error while scanning for viruses with $av");
      $mta->freeze_message($args);
      return 0;
    };
  }

  # Decide what to do with the message
  if (defined $$args{'found_viruses'}) {
    # Log what viruses have been found.
    foreach my $scanner (@{$$args{'virus_scanners'}}) {
      writelog($args,LOG_INFO, "$scanner found:");
      foreach my $virus (@{$$args{'found_viruses'}{$scanner}}) {
	writelog($args,LOG_INFO, " $virus");
      }
    }

    # Tell MTA module to drop the message
    $mta->drop_message($args);

    # Quarantine message
    if ($cfg_quarantine_dir ne 'no') {
      $self->quarantine_message($args);
    }

    # Complain to everybody who cares if policy check failed or a
    # virus was found
    foreach my $notifier (@notifiers) {
      $notifier->notify($args) or
	writelog($args,LOG_CRIT, "Error while notifying $notifier");
    }
  }
  else {
    $mta->accept_message($args);
  }

  # Return successfully
  return 1;
}

sub quarantine_message {
  my $self = shift;
  my $args = shift;
  # quarantine message
  $$args{'quarantine_file'}=sprintf("%.8x-%.4x",time,$$);
  writelog($args,LOG_INFO, "Quarantining infected message to $cfg_quarantine_dir/$$args{'quarantine_file'}");
  
  copy("$$args{'directory'}/email.txt",
       "$cfg_quarantine_dir/$$args{'quarantine_file'}.msg") or do {
	 writelog($args,LOG_CRIT, "Quaranining failed: $!");
       };
  # write part of logfile to quarantine dir.
  my $logfile = IO::File->new(">$cfg_quarantine_dir/".
			      "$$args{'quarantine_file'}.log");
  unless ((defined $logfile) && 
	  ($logfile->print($$args{'log'})) &&
	  ($logfile->close())) {
    writelog($args,LOG_CRIT, "Writing log to quarantine dir failed");
  }
}

# Open file descriptor from
sub cmd_pipe {
  my $args = shift;
  my $handle = IO::Handle->new(); # _from_fd(\*HANDL, 'r');
  my $pid = open($handle, "-|");
  unless (defined $pid ) {
    writelog($args,LOG_ERR, __PACKAGE__.": Can't fork: $!");
    die "Can't fork: $!";
  }
  if ($pid) {           # parent
    return $handle;
  } else {
    # Drop privileges if UID=root
    if ($< == 0) {
      setuid($cfg_uid) or do {
	writelog($args,LOG_ERR, __PACKAGE__.": Can't drop UID to $cfg_uid");
#	die;
      };
      setgid($cfg_gid) or do {
	writelog($args,LOG_ERR, __PACKAGE__.": Can't drop GID to $cfg_gid");
#	die;
      };
    }
    $ENV{PATH} = "/bin:/usr/bin";
    exec @_
      or die "can't exec $_[0]: $!";
  }
}

# Cleanup routine
# Removes working directory
sub cleanup {
  my $self = shift;
  my $args = shift;
  if ($cfg->val('paths','cleanup') eq 'yes') {
    writelog($args,LOG_INFO, __PACKAGE__.": Cleaning up.");
    writelog($args,LOG_INFO, __PACKAGE__.": Removing $$args{'directory'}");
    eval {
      rmtree($$args{'directory'}) or do {
	writelog($args,LOG_CRIT, __PACKAGE__.": Error: Unable to remove directory");
	return 0
      };
      if ($@) {
	writelog($args,LOG_INFO, __PACKAGE__.": Cleanup error: $@");
      }
    };
  }
  # Return successfully
  return 1;
}

1;
