# $Id: CLAM.pm,v 1.1.2.1 2002/05/11 19:29:22 bengen Exp $

#
# Module for CLAM Antivirus
#

package AMAVIS::AV::CLAM;
use strict;

use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;

use vars qw(
	    $cfg_clam_binary
	   );

sub init {
  my $self = shift;
  my $args = shift;
  push @{$$args{'virus_scanners'}}, 'CLAM Anti Virus';
  $cfg_clam_binary = $AMAVIS::cfg->val('CLAM', 'clamscan');
  if ($cfg_clam_binary eq '') {
    writelog($args,LOG_CRIT, "Path to CLAM not specified");
    return 0;
  }
  if (! -x $cfg_clam_binary) {
    writelog($args,LOG_CRIT, "CLAM ($cfg_clam_binary) not executable");
    return 0;
  }
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  # Return successfully
  return 1;
}

sub scan {
  my $self = shift;
  my $args = shift;

  my @output;

  writelog($args,LOG_DEBUG,"Scanning with CLAM");
  writelog($args,LOG_DEBUG,
	   "Output from $cfg_clam_binary --dumb $$args{directory}/parts");
  my $output_handle = cmd_pipe($args, $cfg_clam_binary, 
			       '--stdout',
			       '-r','-w',
			       "$$args{directory}/parts");
  while(<$output_handle>) {
    chomp;
    s/[\000-\037]*//g;
    writelog($args,LOG_DEBUG,$_);
    push @output, $_;
  }
  writelog($args,LOG_DEBUG, "End output from $cfg_clam_binary\n");
  wait;
  my $errval = ($? >> 8);
  writelog($args,LOG_DEBUG,"Return code $errval");
  if ($errval != 0) {
    if ($errval == 1) {
      my %uniq;
      foreach my $output (@output) {
	if ( $output =~ /.*: (.+) FOUND/g ) {
	  unless($uniq{$1}++) {
	    push @{$$args{'found_viruses'}{'CLAM Anti Virus'}}, $1;
	  }
	}
      }
    } else {
      writelog($args,LOG_ERR,"Virus scanner failure: $cfg_clam_binary (error code: $errval)");
      # return ERROR
      return 0;
    }
  }
  # Return successfully
  return 1;
}

1;
