# $Id: Panda.pm,v 1.1.2.2 2002/05/17 06:08:09 bengen Exp $

#
# Support module for Panda Antivirus
#

package AMAVIS::AV::Panda;
use strict;

use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;
use IO::Handle;

use vars qw(
	    $cfg_panda_binary
	   );

sub init {
  my $self = shift;
  my $args = shift;
  push @{$$args{'virus_scanners'}}, 'Panda Software Antivirus';

  $cfg_panda_binary = $AMAVIS::cfg->val('Panda', 'panda');
  if ($cfg_panda_binary eq '') {
    writelog($args, LOG_CRIT, "Path to Panda not specified");
    return 0;
  }
  if (! -x $cfg_panda_binary) {
    writelog($args, LOG_CRIT, "Panda ($cfg_panda_binary) not executable");
    return 0;
  }

  writelog($args, LOG_DEBUG,__PACKAGE__." initialized.\n");
  # Return successfully
  return 1;
}

# Called from AMAVIS.pm. Scanning takes place here.
sub scan {
  my $self = shift;
  my $args = shift;

  my @output;

  writelog($args, LOG_DEBUG,"Scanning with Panda");
  writelog($args, LOG_DEBUG,
	   "Output from ".join(' ',
			       $cfg_panda_binary, 
			       "$$args{directory}/parts", 
			       '-aut', '-eng', '-heu', '-nso',
			       '-aex', '-nor'));
  my $output_handle = cmd_pipe($args, $cfg_panda_binary, 
			       "$$args{directory}/parts", 
			       '-aut', '-eng', '-heu', '-nso',
			       '-aex', '-nor');
  my $numvirus=0;

  while(<$output_handle>) {
    chomp;
    # Escape
    s/\e\133(..)G/\n/g;
    s/\e(.*?)[A-Z,a-z,>]//g;
    # ^O, ^M
    s/\015//g;
    s/\017//g;
    # ^H
    s/\010(\010+?)\010/\010/g;
    s/\010/\n/g;
    push @output, $_;
    writelog($args, LOG_DEBUG, $_);
    if (/Number of files infected\.+:\s*(.+)/gm) {
      $numvirus = $1;
    }
  }
  writelog($args, LOG_DEBUG, "End output from $cfg_panda_binary\n");
  wait;
  my $errval = ($? >> 8);
  writelog($args, LOG_DEBUG,"Return code $errval");
  if ($errval == 0) {
    if($numvirus > 0) {
      my %uniq;
      foreach my $output (@output) {
	if ( $output =~ /Found virus :\s*(.+)/g ) {
	  unless($uniq{$1}++) {
	    push @{$$args{'found_viruses'}{'Panda Software Antivirus'}}, $1;
	  }
	}
      }
    } else {
      writelog($args,LOG_ERR,
	       "Virus scanner failure: Some reason.");
      return 0;
    }
  }
  else {
    writelog($args,LOG_ERR,
	     "Virus scanner failure: Some reason.");
    return 0;
  }
  return 1;
}

1;
