# $Id: Trend.pm,v 1.2.2.1 2002/05/17 06:08:29 bengen Exp $

#
# Trend code for virus scanners
#

package AMAVIS::AV::Trend;
use strict;

use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;

use vars qw(
	    $cfg_trend_binary
	   );

sub init {
  my $self = shift;
  my $args = shift;
  push @{$$args{'virus_scanners'}}, 'Trend';
  $cfg_trend_binary = $AMAVIS::cfg->val('Trend', 'vscan');
  if ($cfg_trend_binary eq '') {
    writelog($args,LOG_CRIT, "Path to Trend not specified");
    return 0;
  }
  if (! -x $cfg_trend_binary) {
    writelog($args,LOG_CRIT, "Trend ($cfg_trend_binary) not executable");
    return 0;
  }
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.\n");
  # Return successfully
  return 1;
}

# Called from AMAVIS.pm. Scanning takes place here.
sub scan {
  my $self = shift;
  my $args = shift;

  my @output;

  writelog($args,LOG_DEBUG,"Scanning with Trend");
  writelog($args,LOG_DEBUG,
	   "Output from ".join(' ',
			       $cfg_trend_binary,
			       '-a',
			       "$$args{directory}/parts"));
  my $output_handle = cmd_pipe($args, 
			       $cfg_trend_binary, 
			       '-a',
			       "$$args{directory}/parts");
  while(<$output_handle>) {
    chomp;
    s/[\000-\037]*//g;
    writelog($args,LOG_DEBUG,$_);
    push @output, $_;
  }
  writelog($args,LOG_DEBUG, "End output from $cfg_trend_binary\n");
  wait;
  my $errval = ($? >> 8);
  writelog($args,LOG_DEBUG,"Return code $errval");
  if ($errval != 0) {
    if ($errval == 1 || $errval == 2) { # no errors, viruses discovered
      my %uniq;
      foreach my $output (@output) {
	if ( $output =~ /Found virus (.+) in/g ) {
	  unless($uniq{$1}++) {
	    push @{$$args{'found_viruses'}{'Trend'}}, $1;
	  }
	}
      }
    } else {
      writelog($args,LOG_ERR,"Virus scanner failure: $cfg_trend_binary (error code: $errval)");
      # return ERROR
      return 0;
    }
  }

  # Return successfully...
  return 1;
}

1;
