# $Id: ARJ.pm,v 1.4.2.1 2002/04/30 14:21:41 bengen Exp $

package AMAVIS::Extract::ARJ;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;

use vars qw(
	    $cfg_unarj_binary
	   );

sub init {
  my $self = shift;
  my $args = shift;
  my $types = shift;
  $cfg_unarj_binary=$AMAVIS::cfg->val('external', 'unarj');
  if (! -x $cfg_unarj_binary) {
    writelog($args,LOG_CRIT,__PACKAGE__.
	     ": unarj binary ($cfg_unarj_binary) not executable");
  }

  $$types{'application/x-arj'}=$self;
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  return 1;
}

sub extract {
  my $self = shift;
  my $args = shift;
  my $filename = shift;

  # Work around DOSism braindamage in unarj.
  if ($filename !~ /.arj$/) {
    $filename.='.';
  }

  my $unpacked_size = 0;
  my @arj_filenames;

  writelog($args,LOG_DEBUG, "Attempting to unpack $filename as ARJ file");


  my $output_handle = cmd_pipe($args, $cfg_unarj_binary, 'l',
			       "$$args{'directory'}/parts/$filename");
  # Skip over 
  while(<$output_handle>) {
    last if (/^----.*----$/);
  }
  while(<$output_handle>) {
    last if (/^----.*----$/);
    # Work around broken behavior with case-sensitiveness.
    (my $member) = /^(\S*)/;
    $member =~ tr [A-Z] [a-z];
    push @arj_filenames,$member;
  }
  while(<$output_handle>) {
  }
  wait;

  foreach (@arj_filenames) {
    if ($$args{'unpacked_files'}++ > $cfg_maxfiles) {
      writelog($args,LOG_ERR, __PACKAGE__.": Unpacking uses too many files");
      return 0;
    }
    writelog($args,LOG_DEBUG, join(' ',$cfg_unarj_binary,
			   'pq',
			   "$$args{'directory'}/parts/$filename",
			   $_));
    my $in_handle=cmd_pipe($args, $cfg_unarj_binary,
			   'pq',
			   "$$args{'directory'}/parts/$filename",
			   $_);
    my $securename=get_secure_filename($args);
    my $out_handle=IO::File->new(">$$args{'directory'}/parts/"
				 .$securename);
    while (my $len=$in_handle->read(my $buffer, 4096)) {
      $out_handle->write($buffer, $len);
      $unpacked_size += $len;
      if ($$args{'unpacked_size'} + $unpacked_size >= $cfg_maxspace) {
	$in_handle->close();
	$out_handle->close();
	writelog($args,LOG_ERR, __PACKAGE__.": Unpacking takes too much space");
	return 0;
      }
    }
    $in_handle->close();
    $out_handle->close();
    $ {$$args{'contents'}}{$securename} = {'original_filename' => $_};
  }
  $$args{'unpacked_size'} += $unpacked_size;
  return 1;
}

1;
