# $Id: Mail.pm,v 1.6.4.2 2002/04/30 14:21:41 bengen Exp $

#
# Module for extracting the plain body or MIME attachments from a mail file
#

package AMAVIS::Extract::Mail;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;
use MIME::Parser;
use IO::File;

sub init {
  my $self = shift;
  my $args = shift;
  my $types = shift;
  $$types{'message/rfc822'}=$self;
  $$types{'message/news'}=$self;
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  return 1;
}


sub extract {
  my $self=shift;
  my $args=shift;
  my $filename=shift;
  writelog($args,LOG_DEBUG, "Attempting to unpack $filename as MIME compliant message");

  my $buffer;
  my $len;
  my $unpacked_size = 0;

  my $parser = new MIME::Parser;

  $parser->output_dir($$args{'directory'});

  my $handle = IO::File->new("<$$args{'directory'}/parts/$filename");

  $parser->extract_nested_messages("REST");

  $parser->ignore_errors(1);
  my $entity = eval { $parser->parse($handle) };
  my $error = ($@ || $parser->last_error);
  if ($error) {
    my %errors = ();
    writelog($args,LOG_ERR,"MIME::Parser error string begin");
    foreach my $errorstring (split /\n/, $error) {
      if ($errorstring !~ /^ *$/) {
	writelog($args,LOG_ERR,"$errorstring");
	$errors{$errorstring}++;
      };
    }
    writelog($args,LOG_ERR,"MIME::Parser error string end");

    foreach (keys %errors) {
      if ($_ !~ /^error:\s*unexpected end of header$/) {
        # Return unsuccessfully
        return 0;
      }
    }
    writelog($args,LOG_ERR,"MIME::Parser error is not fatal, proceeding further");
  }

  # Have a look at all parts of $entity, starting with $entity itself
  foreach my $part ($entity->parts_DFS()) {
    my $in_handle=$part->open('r') or next;
    if ($$args{'unpacked_files'}++ > $cfg_maxfiles) {
      writelog($args,LOG_ERR, __PACKAGE__.": Unpacking uses too many files");
      return 0;
    }
    # Write entity to file unless empty
    my $securename=get_secure_filename($args);
    my $out_handle=IO::File->new(">$$args{'directory'}/parts/"
				 .$securename);
    while ($len=$in_handle->read($buffer, 4096)) {
      $out_handle->write($buffer, $len);
      $unpacked_size += $len;
      if ($$args{'unpacked_size'} + $unpacked_size >= $cfg_maxspace) {
	$in_handle->close();
	$out_handle->close();
	writelog($args,LOG_ERR, __PACKAGE__.": Unpacking takes too much space");
	return 0;
      }
    }
    $in_handle->close();
    $out_handle->close();
    $ {$$args{'contents'}}{$securename} = {};
  }
  $parser->filer->purge;
  $$args{'unpacked_size'} += $unpacked_size;
  return 1;
}

1;
