# $Id: ZOO.pm,v 1.7.4.1 2002/04/30 14:21:41 bengen Exp $

#
# Module for extracting ZOO files
#

package AMAVIS::Extract::ZOO;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;

use vars qw(
	    $zoo_binary
	    );

sub init {
  my $self = shift;
  my $args = shift;
  my $types = shift;
  $zoo_binary=$AMAVIS::cfg->val('external', 'zoo');
  $$types{'application/x-zoo'}=$self;
  if (! -x  $zoo_binary) {
    writelog($args,LOG_CRIT, __PACKAGE__.
	     ": ZOO binary ($zoo_binary) not executable");
    return 0;
  }
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  return 1;
}

sub extract ( $ ) {
  my $self = shift;
  my $args = shift;
  my $filename = shift;
  my $unpacked_size = 0;

  my $buffer;
  my $len;

  writelog($args,LOG_DEBUG, "Attempting to unpack $filename as ZOO file");

  # Zoo needs extension of .zoo!
  symlink("$$args{'directory'}/parts/$filename", 
	  "$$args{'directory'}/parts/$filename.zoo");

  # This is safe. No filenames specified from outside
  my @list;
  my $output_handle = cmd_pipe($args, $zoo_binary, 'lf1q',
			       "$$args{'directory'}/parts/$filename.zoo");
  while (<$output_handle>) {
    chop;
    push @list, $_;
  }

  foreach (@list) {
    # Ignore directories
    unless (/.*\/$/) {
      if ($$args{'unpacked_files'}++ > $cfg_maxfiles) {
	writelog($args,LOG_ERR, __PACKAGE__.": Unpacking uses too many files");
	return 0;
      }
      my $in_handle=cmd_pipe($args, $zoo_binary,
			     'xpqqq:',
			     "$$args{'directory'}/parts/$filename.zoo",
			     $_);
      my $securename=get_secure_filename($args);
      my $out_handle=IO::File->new(">$$args{'directory'}/parts/"
				   .$securename);

      while ($len=$in_handle->read($buffer, 4096)) {
	$out_handle->write($buffer, $len);
	$unpacked_size += $len;
	if ($$args{'unpacked_size'} + $unpacked_size >= $cfg_maxspace) {
	  $in_handle->close();
	  $out_handle->close();
	  writelog($args,LOG_ERR, __PACKAGE__.": Unpacking takes too much space");
	  return 0;
	}
      }

      $ {$$args{'contents'}}{$securename} = {'original_filename' => $_};

    }
  }
  unlink("$$args{'directory'}/parts/$filename.zoo");
  $$args{'unpacked_size'} += $unpacked_size;
  return 1;
}

1;
