# $Id: Exim.pm,v 1.9.4.1 2002/04/30 14:21:48 bengen Exp $

#
# MTA module for "regular" Exim setup
#

package AMAVIS::MTA::Exim;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;
use IO::File;
use File::Path;

use File::Copy;
use Sys::Hostname;


use vars qw(
	    $cfg_exim_binary
	    $cfg_exim_args
	    $cfg_x_header
	    $cfg_x_header_tag
	    $cfg_x_header_line
	    $cfg_problem_dir
	   );

sub init {
  my $self = shift;
  my $args = shift;
  $cfg_exim_binary = $AMAVIS::cfg->val('Exim', 'exim');
  $cfg_exim_args = $AMAVIS::cfg->val('Exim', 'args');
  if ($cfg_exim_binary eq '') {
    writelog($args,LOG_CRIT, "Location of exim binary not specified");
    return 0;
  }
  if (! -x $cfg_exim_binary) {
    writelog($args,LOG_CRIT, "$cfg_exim_binary not executable");
    return 0;
  }

  $cfg_problem_dir = $AMAVIS::cfg->val('Exim', 'problem dir');
  if ($cfg_problem_dir eq '') {
    writelog($args,LOG_CRIT, "Exim problem directory not specified");
    return 0;
  }
  $cfg_exim_binary = $AMAVIS::cfg->val('Exim', 'exim');
  $cfg_exim_args = $AMAVIS::cfg->val('Exim', 'args');
  if ($AMAVIS::cfg->val('global', 'x-header') eq 'true') {
    $cfg_x_header = 1
  };
  $cfg_x_header_tag = $AMAVIS::cfg->val('global', 'x-header-tag');
  $cfg_x_header_line = $AMAVIS::cfg->val('global', 'x-header-line');
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  # Return successfully
  return 1;
}

# Create temp dir and write mail
sub get_directory {
  my $self = shift;
  my $args = shift;

  # Create temp directory. Try creating $prefix[date]-[pid] 
  # If unsuccessful after 10 tries, abort
  my $prefix = $AMAVIS::cfg->val('paths', 'unpack').'amavis-unpack-';
  my $i = 0;
  my $message_id;
  while (1) {
    $message_id = sprintf("%.8x-%.4x",time,$$);
    unless (defined mkpath ($prefix.$message_id, 0, 0770)) {
      if (++$i > 10) {
	return 0;
      }
      else {
	next;
      }
    }
    last;
  }
  $$args{'message_id'}=$message_id;
  my $directory = $prefix.$message_id;
  mkdir "$directory/parts", 0777;

  # Open message file that is later going to be disected and scanned
  my $output = IO::File->new("+>$directory/email.txt");

  # Get message from STDIN
  my $headers=1;
  $$args{'headers'}.='';
  while (<STDIN>) {
    if (/^ *$/) {
      $headers=0;
    }
    if ($headers==1) {
      $$args{'headers'}.=$_;
    }
    print $output $_;
  }

  if ($#ARGV < 1) {
    writelog($args,LOG_ERR, __PACKAGE__.": Missing arguments to exim");
    return 0;
  }

  # FIXME: There should be a way to pass the message id here.
  # Maybe amavis -m $m_id
  shift @ARGV if $ARGV[0] eq "-f" ; #ignore "-f"
  $$args{'sender'} = shift @ARGV;
  $$args{'sender'} = "<>" if (!$$args{'sender'});
  shift @ARGV if $ARGV[0] eq "-d" ; #ignore "-d"
  push @{$$args{'recipients'}}, @ARGV;
  writelog($args,LOG_DEBUG,__PACKAGE__.": Sender is $$args{'sender'}");
  writelog($args,LOG_DEBUG,__PACKAGE__.": Recipients are "
	   .join(', ',@{$$args{'recipients'}}));

  # Message file has been written, reset file pointer and put it into
  # the record.
  $output->seek(0,0);
  $$args{'filehandle'} = $output;
  $$args{'directory'} = $directory;

  # Return successfully
  return 1;
}

# Generate a copy of the scanned message and pipe it to mailer.
sub accept_message {
  my $self = shift;
  my $args = shift;
  writelog($args,LOG_INFO, __PACKAGE__.": Accepting message");
  my @cfg_exim_args;

  push @cfg_exim_args, split(/\s+/,$cfg_exim_args);
  push @cfg_exim_args, $$args{'sender'};
  push @cfg_exim_args, @{$$args{'recipients'}};

  open(MAIL, "|-") || exec($cfg_exim_binary, @cfg_exim_args);
  if ($cfg_x_header) {
    print MAIL "$cfg_x_header_tag: $cfg_x_header_line\n";
  }
  while (my $line=$$args{'filehandle'}->getline()) {
    print MAIL $line;
  }
  close(MAIL);

  if ($? != 0) {
    writelog($args,LOG_ERR, __PACKAGE__.": $cfg_exim_binary exited with ".($?>>8));
  }

  $$args{'status'} = 'accept';

  # Return successfully
  return 1;
}

# Just "forget" the mail.
sub drop_message {
  my $self = shift;
  my $args = shift;
  # Doing nothing should suffice
  writelog($args,LOG_INFO, __PACKAGE__.": Dropping message");

  # Return successfully
  return 1;
}

sub freeze_message {
  my $self = shift;
  my $args = shift;
  writelog($args,LOG_INFO, __PACKAGE__.": Freezing message");

  # Instead of freezing, drop the message, but store a copy in
  # $cfg_problem_dir -- analogous to AMAVIS::quarantine_message()

  writelog($args,LOG_DEBUG, "Putting message into problem dir");
  if (copy("$$args{'directory'}/email.txt",
	   "$cfg_problem_dir/$$args{'message_id'}.msg")) {
    # write part of logfile to quarantine dir.
    my $logfile = IO::File->new(">$cfg_problem_dir/".
				"$$args{'message_id'}.log");
    unless ((defined $logfile) && 
	    ($logfile->print($$args{'log'})) &&
	    ($logfile->close())) {
      writelog($args,LOG_CRIT, "Writing log into problem dir failed");
    }
    my $hostname=hostname();
    my $error_address=$AMAVIS::cfg->val('Notify', 'admin');
    my $msg=<<"EOF";
Subject: Message put in AMaViS-Exim problem directory

Message $$args{'message_id'}
has been put into AMaViS-Exim problem directory as
$hostname:$cfg_problem_dir/$$args{'message_id'}.msg.

The corresponding logfile has been written to
$hostname:$cfg_problem_dir/$$args{'message_id'}.log.

EOF
    $self->send_message($args, $msg, $error_address, $error_address);
  }
  else {
    writelog($args,LOG_ERR, "Unable to put message into problem dir $cfg_problem_dir. Freezing message.");
    return 0;
  }

  # Return successfully
  return 1;
}

# Called from Notify::*.pm, i.e. for sending warining messages
sub send_message {
  my $self = shift;
  my $args = shift;
  my $message = shift;
  my $sender = shift;
  my @recipients = @_;
  writelog($args,LOG_DEBUG, __PACKAGE__.": Sending mail from $sender to ".
	   join(', ',@recipients));

  my @cfg_exim_args;

  push @cfg_exim_args, split(/\s+/,$cfg_exim_args);
  push @cfg_exim_args, $sender;
  push @cfg_exim_args, @recipients;

  writelog($args,LOG_DEBUG, __PACKAGE__.": Running $cfg_exim_binary ".
	   join(' ',@cfg_exim_args));

  open(MAIL, "|-") || exec($cfg_exim_binary, @cfg_exim_args);
  print MAIL $message;
  close(MAIL);

  if ($? != 0) {
    writelog($args,LOG_ERR,
	     __PACKAGE__.": $cfg_exim_binary @cfg_exim_args exited with ".($?>>8));
  }

  # Return successfully
  return 1;
}

1;
