# $Id: Recipients.pm,v 1.11.2.2 2002/05/07 00:56:04 bengen Exp $

package AMAVIS::Notify::Recipients;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;

use vars qw(
	    $cfg_sender
	    $cfg_internal_domain
	   );

sub init {
  my $self = shift;
  my $args = shift;
  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  $cfg_sender = $AMAVIS::cfg->val('Notify', 'mail from');
  $cfg_sender = 'postmaster' unless defined($cfg_sender);
  $cfg_internal_domain = $AMAVIS::cfg->val('Notify', 'local domain');
  # Return successfully
  return 1;
}

sub notify {
  my $self = shift;
  my $args = shift;
  my @recipients;

  my $orig_sender = $$args{'sender'};
  if ($orig_sender eq '') {
    $orig_sender = '(empty address)';
  }

  foreach my $recipient (@{$$args{'recipients'}}) {
    # Skip if recipient is not in $cfg_internal_domain.
    unless ($recipient=~/\@$cfg_internal_domain(?:[> ].*)?/) {
      writelog($args,LOG_DEBUG, "Not warning recipient $recipient: non-local user");
      next;
    }
    writelog($args,LOG_DEBUG, "Warning recipient: $recipient.");
    push @recipients, $recipient;
  }
  # Success if nobody has to be warned.
  return 1 if ($#recipients == -1);
  my $recipients=join(' ', @recipients);

  my $message = <<"EOF";
From: $cfg_sender
To: $recipients
Subject: VIRUS IN MAIL FOR YOU FROM $orig_sender.

Our virus checker has found potentially malicious code in a mail from
$orig_sender addressed to you. Delivery has been stopped. For further
questions, please contact $cfg_sender.

EOF
  if (defined $$args{'quarantine_file'}) {
    $message.= << "EOF";

The message has been quarantined as $$args{'quarantine_file'}.

EOF
  };
  foreach my $scanner (@{$$args{'virus_scanners'}}) {
    $message.="\n$scanner found:\n";
    foreach my $virus (@{$$args{'found_viruses'}{$scanner}}) {
      $message.=" $virus\n";
    }
  }

  $message.="\n".('-'x 72)."\nMessage headers follow:\n";
  $message.=$$args{'headers'};

  $AMAVIS::mta->send_message($args, $message, '<>', @recipients)
    or writelog($args,LOG_ERR,"Error notifying reipients: $recipients");;
  return 1;
}

1;
